package student.tasks;

import java.awt.Color;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import javax.vecmath.Point2d;

import org.jgrapht.GraphPath;

import student.refsolution.rrtstar.Domain;
import student.refsolution.rrtstar.Extension;
import student.refsolution.rrtstar.ExtensionEstimate;
import student.refsolution.rrtstar.RRTStarPlanner;
import cz.agents.alite.pahtactical.vis.PathLayer;
import cz.agents.alite.pahtactical.vis.Polygons2dLayer;
import cz.agents.alite.pahtactical.vis.VehicleLayer;
import cz.agents.alite.tactical.util.Polygon2d;
import cz.agents.alite.vis.VisManager;
import cz.agents.alite.vis.layer.common.ColorLayer;
import cz.agents.alite.vis.layer.common.VisInfoLayer;
import dubins.DubinsCurve;
import dubins.Point2dYaw;

public class Task4 {
	
	/**
	 *  This method should return a "reasonable" path for a vehicle with given footprint and minimum turn radius
	 *  that begins at start configuration and ends at the goal configuration and the vehicle will avoid collisions 
	 *  with given obstacles. 
	 *  
	 *  Use RRT* algorithm with Dubin's curve extensions.
	 *
	 *  @return poly-line representation of the found path
	 */
	public static Point2d[] findPath(
			final Point2dYaw start, final Point2dYaw end, final Collection<Polygon2d> obstacles, 
			final Polygon2d footprint, final double rho) {
		
		// put your solution here
	
		return new Point2d[0];
	}

	public static void main(String[] args) {	
		
		Polygon2d footprint = getFootprint();
		
		Point2dYaw start = new Point2dYaw(0, -200, 0);
		Point2dYaw end   = new Point2dYaw(0, 250, Math.PI/3);
		
		Collection<Polygon2d> polygons = new LinkedList<Polygon2d>();
		polygons.add(getRect(0, 0, 200, 200));
		polygons.add(getRect(100, 100, 500, 10));
		polygons.add(getRect(-100, 100, 100, 170));

		VisManager.registerLayer(ColorLayer.create(Color.WHITE));
		VisManager.registerLayer(Polygons2dLayer.create(polygons, Color.BLACK, 2));
		VisManager.registerLayer(VehicleLayer.create(footprint, start, Color.GREEN));
		VisManager.registerLayer(VehicleLayer.create(footprint, end, Color.RED));
		
		VisManager.registerLayer(VisInfoLayer.create());
		
        VisManager.setInitParam("PAH Trajectory Planning Task", 1024, 768);
        VisManager.init();
        
        Point2d[] path = findPath(start, end, polygons, footprint, 40);
        
        VisManager.registerLayer(PathLayer.create(path, Color.BLUE, 3));
	}
	
	public static Polygon2d getRect(int x, int y, int w, int h) {
		return new Polygon2d(new Point2d[]{
				new Point2d(x - w/2, y -h/2),
				new Point2d(x + w/2, y -h/2),
				new Point2d(x + w/2, y +h/2),
				new Point2d(x - w/2, y +h/2)});
	}
	
	public static Polygon2d getFootprint() {
		int h = 20;
		int wFront = 25;
		int wBack = 8;
		return new Polygon2d(new Point2d[]{
				new Point2d(-wBack,  -h/2),
				new Point2d(+wFront, -h/2),
				new Point2d(+wFront,  +h/2),
				new Point2d(-wBack, +h/2)});
	}
	
}
